"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = generate;
const path_1 = require("path");
const mem_fs_1 = require("mem-fs");
const mem_fs_editor_1 = require("mem-fs-editor");
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const project_access_1 = require("@sap-ux/project-access");
const config_1 = require("./config");
const file_1 = require("./file");
const constants_1 = require("./constants");
/**
 * Writes the template to the memfs editor instance.
 *
 * @param basePath - the base path
 * @param abapDeployConfig - the abap deploy config
 * @param options - the deploy config options
 * @param fs - the memfs editor instance
 * @returns the updated memfs editor instance
 */
async function generate(basePath, abapDeployConfig, options, fs) {
    const abapConfig = (0, cloneDeep_1.default)(abapDeployConfig);
    if (!fs) {
        fs = (0, mem_fs_editor_1.create)((0, mem_fs_1.create)());
    }
    // base config
    const baseConfigFile = options?.baseFile ?? project_access_1.FileName.Ui5Yaml;
    const baseConfig = await (0, project_access_1.readUi5Yaml)(basePath, baseConfigFile, fs);
    const isLib = baseConfig.getType() === 'library';
    const baseConfigPath = (0, path_1.join)(basePath, baseConfigFile);
    (0, config_1.updateBaseConfig)(isLib, baseConfigPath, baseConfig, fs);
    // deploy config
    const deployConfigFile = options?.deployFile ?? project_access_1.FileName.UI5DeployYaml;
    const deployFilePath = (0, path_1.join)(basePath, deployConfigFile);
    const deployConfig = await (0, config_1.getDeployConfig)(abapConfig, baseConfig);
    fs.write(deployFilePath, deployConfig.toString());
    // package.json
    // deploy script
    const deployScript = `${constants_1.BUILD_SCRIPT} && ${constants_1.DEPLOY_SCRIPT} --config ${deployConfigFile} && ${constants_1.RIMRAF} archive.zip`;
    await (0, project_access_1.updatePackageScript)(basePath, 'deploy', deployScript, fs);
    // undeploy script
    const undeployScript = `${constants_1.BUILD_SCRIPT} && ${constants_1.UNDEPLOY_SCRIPT} --config ${deployConfigFile}`;
    await (0, project_access_1.updatePackageScript)(basePath, 'undeploy', undeployScript, fs);
    // test mode script
    const deployTestModeScript = `${constants_1.BUILD_SCRIPT} && ${constants_1.DEPLOY_SCRIPT} --config ${deployConfigFile} --testMode true`;
    await (0, project_access_1.updatePackageScript)(basePath, 'deploy-test', deployTestModeScript, fs);
    // dependencies
    await (0, project_access_1.addPackageDevDependency)(basePath, constants_1.RIMRAF, constants_1.RIMRAF_VERSION, fs);
    if (isLib) {
        // ui5 repo ignore file
        await (0, project_access_1.addPackageDevDependency)(basePath, constants_1.UI5_TASK_FLATTEN_LIB, constants_1.UI5_TASK_FLATTEN_LIB_VERSION, fs);
        (0, file_1.addUi5Dependency)(fs, basePath, constants_1.UI5_TASK_FLATTEN_LIB);
        const libPath = await (0, file_1.getLibraryPath)(basePath);
        await (0, file_1.writeUi5RepositoryIgnore)(fs, libPath);
    }
    else {
        // ui5 repo file
        const webappPath = await (0, project_access_1.getWebappPath)(basePath);
        await (0, file_1.writeUi5RepositoryFiles)(fs, webappPath);
    }
    return fs;
}
//# sourceMappingURL=index.js.map